from dotenv import load_dotenv

load_dotenv()
from datetime import timedelta
from pathlib import Path
import sys
import os


BASE_DIR = Path(__file__).resolve().parent.parent


SECRET_KEY = os.getenv('SECRET_KEY') 

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.getenv('DEBUG', 'False').lower() == 'true'

ALLOWED_HOSTS = [host.strip() for host in os.getenv("ALLOWED_HOSTS", "").split(",") if host]


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'users.apps.UsersConfig',

    'rest_framework',
    'rest_framework_simplejwt',
    'rest_framework_simplejwt.token_blacklist',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]


AUTH_USER_MODEL = 'users.User'




CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.filebased.FileBasedCache",
        "LOCATION": "/tmp/django_cache",  # You can customize this
    }
}




REST_FRAMEWORK = {
    
    'DEFAULT_AUTHENTICATION_CLASSES': (
        
        'rest_framework_simplejwt.authentication.JWTAuthentication',
    ),
    'EXCEPTION_HANDLER': 'coreuser.utils.custom_exception_handler',

    'DEFAULT_THROTTLE_CLASSES': [
        'rest_framework.throttling.ScopedRateThrottle',
    ],
     "DEFAULT_THROTTLE_RATES": {       
        "login": "2/minute",      
        "register": "5/minute",
        "logout": "5/minute",     
    },
    
}


SIMPLE_JWT = {
    "ACCESS_TOKEN_LIFETIME": timedelta(minutes=5),
    "REFRESH_TOKEN_LIFETIME": timedelta(days=1),
    "ROTATE_REFRESH_TOKENS": True,
    "BLACKLIST_AFTER_ROTATION": True,
    "UPDATE_LAST_LOGIN": False,

    "ALGORITHM": "HS256",
    "SIGNING_KEY": SECRET_KEY,
    "VERIFYING_KEY": "",
    "AUDIENCE": None,
    "ISSUER": None,
    "JSON_ENCODER": None,
    "JWK_URL": None,
    "LEEWAY": 0,

    "AUTH_HEADER_TYPES": ("Bearer",),
    "AUTH_HEADER_NAME": "HTTP_AUTHORIZATION",
    "USER_ID_FIELD": "id",
    "USER_ID_CLAIM": "user_id",
    "USER_AUTHENTICATION_RULE": "rest_framework_simplejwt.authentication.default_user_authentication_rule",

    "AUTH_TOKEN_CLASSES": ("rest_framework_simplejwt.tokens.AccessToken",),
    "TOKEN_TYPE_CLAIM": "token_type",
    "TOKEN_USER_CLASS": "rest_framework_simplejwt.models.TokenUser",

    "JTI_CLAIM": "jti",

    "SLIDING_TOKEN_REFRESH_EXP_CLAIM": "refresh_exp",
    "SLIDING_TOKEN_LIFETIME": timedelta(minutes=5),
    "SLIDING_TOKEN_REFRESH_LIFETIME": timedelta(days=1),

    "TOKEN_OBTAIN_SERIALIZER": "rest_framework_simplejwt.serializers.TokenObtainPairSerializer",
    "TOKEN_REFRESH_SERIALIZER": "rest_framework_simplejwt.serializers.TokenRefreshSerializer",
    "TOKEN_VERIFY_SERIALIZER": "rest_framework_simplejwt.serializers.TokenVerifySerializer",
    "TOKEN_BLACKLIST_SERIALIZER": "rest_framework_simplejwt.serializers.TokenBlacklistSerializer",
    "SLIDING_TOKEN_OBTAIN_SERIALIZER": "rest_framework_simplejwt.serializers.TokenObtainSlidingSerializer",
    "SLIDING_TOKEN_REFRESH_SERIALIZER": "rest_framework_simplejwt.serializers.TokenRefreshSlidingSerializer",
}




ROOT_URLCONF = 'coreuser.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'coreuser.wsgi.application'


# Database
# https://docs.djangoproject.com/en/5.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}


# Password validation
# https://docs.djangoproject.com/en/5.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'users.validators.CustomPasswordValidator', 
    },
]



# Internationalization
# https://docs.djangoproject.com/en/5.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.2/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/5.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'



import sys

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,

    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
        },
    },

    'formatters': {
        'default': {
            'format': '[{asctime}] {levelname} in {name}: {message}',
            'style': '{',
        },
    },

    'root': {
        'handlers': ['console'],
        'level': 'WARNING',
    },

    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
        'users': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': False,
        },
    }
}




from django.db.models.signals import post_save
from django.dispatch import receiver
from django.contrib.auth.models import Group
from django.contrib.auth import get_user_model

User = get_user_model()



@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        regular_user, created=  Group.objects.get_or_create(name='Regular User')
        instance.groups.add(regular_user)


from django.shortcuts import render
from rest_framework import status
from rest_framework.response import Response
from rest_framework.decorators import api_view, permission_classes,throttle_classes
from rest_framework.permissions import AllowAny,IsAuthenticated
from .throttles import LoginThrottle, RegisterThrottle, LogoutThrottle
from .serializers import RegistrationSerializer,LoginSerializer,LogoutSerializer
from rest_framework_simplejwt.tokens import RefreshToken,TokenError

from rest_framework_simplejwt.serializers import TokenRefreshSerializer

import logging

logger = logging.getLogger(__name__)




from rest_framework.exceptions import ValidationError

@api_view(['POST'])
@permission_classes([AllowAny])
@throttle_classes([RegisterThrottle])
def register(request):
    request.throttle_scope = 'register'
    serializer = RegistrationSerializer(data=request.data)
    try:
        serializer.is_valid(raise_exception=True)
        user = serializer.save()
        logger.info(f"New user registered: {user.email}")
        return Response({
            "user": serializer.data,
            "message": "User created successfully."
        }, status=status.HTTP_201_CREATED)

    except ValidationError as ve:
        logger.warning(f"Validation error during registration: {ve}")
        return Response({
            'status': 'error',
            'message': 'Validation failed',
            'errors': ve.detail  
        }, status=status.HTTP_400_BAD_REQUEST)

    except Exception as e:
        logger.exception("Error during user registration")
        return Response({
            'status': 'error',
            'message': "Something went wrong on our end."
        }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)





@api_view(['POST'])
@permission_classes([AllowAny])
@throttle_classes([LoginThrottle])
def login_view(request):
    request.throttle_scope = 'login'
    try:
        serializer = LoginSerializer(data=request.data)
        if serializer.is_valid(raise_exception=True):
            user = serializer.validated_data['user']
            refresh = RefreshToken.for_user(user)
            logger.info(f"[LOGIN] User logged in: {user.email}")
            return Response({
                'refresh': str(refresh),
                'access': str(refresh.access_token),
                'user':{
                    'id':str(user.id),
                    'email': user.email,
                    'full_name': user.full_name
                }
            }, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    except Exception as e:
        logger.exception("[LOGIN ERROR] Exception during user login")
        return Response({
            'message': 'something went wrong on our end'
        }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    








@api_view(['POST'])
@permission_classes([AllowAny])
def custom_token_refresh_view(request):
    try:
        serializer = TokenRefreshSerializer(data=request.data)
        if serializer.is_valid(raise_exception=True):
            access = serializer.validated_data['access']

            return Response({
                "access": access
            }, status=status.HTTP_200_OK)

    except TokenError as e:
        logger.warning(f"[REFRESH ERROR] Token error: {str(e)}")
        return Response({
            "message": "Invalid refresh token."
        }, status=status.HTTP_401_UNAUTHORIZED)
    except Exception as e:
        logger.exception("[REFRESH ERROR] Unexpected error during token refresh")
        return Response({
            "message": "Something went wrong on our end."
        }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)






@api_view(['POST'])
@permission_classes([IsAuthenticated])
@throttle_classes([LogoutThrottle])
def logout_view(request):
    request.throttle_scope = 'logout'
    try:
        serializer = LogoutSerializer(data=request.data)
        if serializer.is_valid(raise_exception=True):
            serializer.save()
            logger.info(f"[LOGOUT] User logged out: {request.user.email}")
            return Response({
                'message': f'Successfully logged out {request.user.email}'
            }, status=status.HTTP_200_OK)
    except Exception as e:
        logger.exception("[LOGOUT ERROR] Exception during user logout")
        return Response({
            'message': 'Something went wrong on our end.'
        }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



from django.db import models
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin,Group
import uuid


# Create your models here.
class CustomUserManager(BaseUserManager):
    def create_user(self, email,first_name,last_name, password=None):
        """
        Creates, saves, and returns a User with the given email, first name, last name, and password.
        """
        if not email:
            raise ValueError('The Email field must be set')
        if not first_name:
            raise ValueError('The First Name field must be set')
        if not last_name:
            raise ValueError('The Last Name field must be set')
        

        email = self.normalize_email(email).lower()
        user = self.model(email=email,first_name=first_name, last_name=last_name)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, first_name, last_name, password=None):
        user = self.create_user(email=email, first_name=first_name, last_name=last_name, password=password)
        user.is_superuser = True
        user.is_staff = True
        user.is_verified = True
        user.save(using=self._db)
        return user
    


AUTH_PROVIDERS = [
    ('email', 'Email'),
    ('google', 'Google'),
    ('facebook', 'Facebook'),
]


class User(AbstractBaseUser, PermissionsMixin):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    email = models.EmailField(max_length=255, unique=True)
    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)
    is_superuser = models.BooleanField(default=False)
    is_verified = models.BooleanField(default=False)
    groups = models.ManyToManyField(Group, blank=True)
    auth_provider = models.CharField(max_length=50,choices=AUTH_PROVIDERS,default='email'
)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    last_login = models.DateTimeField(blank=True, null=True)
    



    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['first_name', 'last_name']

    objects = CustomUserManager()


    def __str__(self):
        return f"{self.first_name} {self.last_name}"
    
    @property
    def full_name(self):
        return f"{self.first_name} {self.last_name}"
    
    class Meta:
        verbose_name = 'user'
        verbose_name_plural = 'users'
        ordering = ['-created_at']
        indexes =[
            models.Index(fields = ['email'])
        ]


    def save(self, *args, **kwargs):
        self.email = self.email.lower()
        super().save(*args, **kwargs)
       




from rest_framework import serializers
from django.contrib.auth.password_validation import validate_password
from rest_framework_simplejwt.tokens import RefreshToken, TokenError
from django.contrib.auth import get_user_model

User = get_user_model()


class RegistrationSerializer(serializers.ModelSerializer):
    password = serializers.CharField(
        required=True,
        write_only=True,
        validators=[validate_password]
    )
    password2 = serializers.CharField(required=True, write_only=True)

    class Meta:
        model = User
        fields = ['id', 'first_name', 'last_name', 'email', 'password', 'password2']
        extra_kwargs = {
            'first_name': {'required': True},
            'last_name': {'required': True},
            'email': {'required': True, 'validators': []},  
        }

    def validate(self, attrs):
        if attrs['password'] != attrs['password2']:
            raise serializers.ValidationError("Passwords do not match.")
        return attrs

    def validate_email(self, value):
        value = value.strip().lower()
        if User.objects.filter(email__iexact=value).exists():
            raise serializers.ValidationError("Email is already in use.")
        return value

    def validate_first_name(self, value):
        if not value.isalpha():
            raise serializers.ValidationError("First name must contain only letters.")
        return value

    def validate_last_name(self, value):
        if not value.isalpha():
            raise serializers.ValidationError("Last name must contain only letters.")
        return value

    def create(self, validated_data):
        validated_data.pop('password2')
        return User.objects.create_user(**validated_data)


class LoginSerializer(serializers.Serializer):
    email = serializers.EmailField(required=True)
    password = serializers.CharField(required=True, write_only=True)

    def validate(self, attrs):
        email = attrs.get('email', '').strip().lower()
        password = attrs.get('password')

        if not email or not password:
            raise serializers.ValidationError("Email and password are required.")

        user = User.objects.filter(email__iexact=email).first()
        if user is None or not user.check_password(password):
            raise serializers.ValidationError("Invalid email or password.")

        attrs['user'] = user
        return attrs





class LogoutSerializer(serializers.Serializer):
    refresh = serializers.CharField()

    def validate_refresh(self, value):
        value = value.strip()
        if not value:
            raise serializers.ValidationError("Refresh token cannot be empty.")

        try:
            RefreshToken(value)
        except TokenError:
            raise serializers.ValidationError("Invalid refresh token.")

        return value

    def save(self, **kwargs):
        try:
            token = RefreshToken(self.validated_data['refresh'])
            token.blacklist()
        except TokenError as e:
            raise serializers.ValidationError(f"Token blacklisting failed: {str(e)}")



import re
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


class CustomPasswordValidator:
    """
    Validates that a password meets the following criteria:
    - At least 10 characters long
    - Contains at least one uppercase letter
    - Contains at least one lowercase letter
    - Contains at least one digit
    - Contains at least one special character
    - Does not contain user's email, first name, or last name
    """

    def validate(self, password, user=None):
        if len(password) < 8:
            raise ValidationError(
                _("This password is too short. It must contain at least 8 characters."),
                code='password_too_short',
            )

        if not re.search(r'[A-Z]', password):
            raise ValidationError(
                _("This password must contain at least one uppercase letter."),
                code='password_no_uppercase',
            )

        if not re.search(r'[a-z]', password):
            raise ValidationError(
                _("This password must contain at least one lowercase letter."),
                code='password_no_lowercase',
            )

        if not re.search(r'[0-9]', password):
            raise ValidationError(
                _("This password must contain at least one digit."),
                code='password_no_digit',
            )

        if not re.search(r'[!@#$%^&*(),.?":{}|<>]', password):
            raise ValidationError(
                _("This password must contain at least one special character (!@#$%^&*(),.?\":{}|<>)."),
                code='password_no_special',
            )

        if user:
            email = user.email.lower() if user.email else ''
            first_name = user.first_name.lower() if user.first_name else ''
            last_name = user.last_name.lower() if user.last_name else ''
            password_lower = password.lower()

            if email and email in password_lower:
                raise ValidationError(
                    _("The password cannot contain your email address."),
                    code='password_contains_email',
                )
            if first_name and first_name in password_lower:
                raise ValidationError(
                    _("The password cannot contain your first name."),
                    code='password_contains_first_name',
                )
            if last_name and last_name in password_lower:
                raise ValidationError(
                    _("The password cannot contain your last name."),
                    code='password_contains_last_name',
                )

    def get_help_text(self):
        return _(
            "Your password must be at least 10 characters long and include an uppercase letter, "
            "a lowercase letter, a digit, and a special character. It must not contain your name or email address."
        )


from django.apps import AppConfig


class UsersConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'users'

    def ready(self):
        # Import signals to ensure they are registered
        import users.signals





# core/utils.py or core/exception_handlers.py

from rest_framework.views import exception_handler
from rest_framework.response import Response
from rest_framework import status
from rest_framework.exceptions import Throttled
import logging

logger = logging.getLogger(__name__)


def custom_exception_handler(exc, context):
    """
    Custom global exception handler for DRF.
    Logs errors and returns a consistent response format.
    """
    # Call REST framework's default exception handler first
    response = exception_handler(exc, context)

    view = context.get('view', None)
    request = context.get('request', None)
    view_name = view.__class__.__name__ if view else 'UnknownView'

    if isinstance(exc, Throttled):
        # Determine user info (if available)
        user_info = 'Anonymous'
        if request and hasattr(request, 'user') and request.user.is_authenticated:
            user_info = request.user.email

        logger.warning(
            f"[THROTTLE] {user_info} exceeded rate limit on {view_name}. "
            f"Retry after {exc.wait} seconds."
        )

        return Response({
            'status': 'error',
            'message': 'Rate limit exceeded. Please try again later.',
            'retry_after_seconds': exc.wait
        }, status=status.HTTP_429_TOO_MANY_REQUESTS)

    # Log all other handled exceptions
    if response is not None:
        logger.error(f"Exception in {view_name}: {str(exc)}")

        custom_response = {
            'status': 'error',
            'message': 'Request failed',
            'errors': response.data
        }
        return Response(custom_response, status=response.status_code)

    # For unhandled exceptions (500)
    logger.exception(f"[UNHANDLED EXCEPTION] {str(exc)}")
    return Response({
        'status': 'error',
        'message': 'Internal server error',
        'errors': str(exc)
    }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



from django.urls import path
from . import views


urlpatterns = [
    path('registration/', views.register, name='register'),
    path('login/', views.login_view, name='login'),
    path('token/refresh/', views.custom_token_refresh_view, name='token_refresh'),
    path('logout/', views.logout_view, name='logout'),
]



# Generated by Django 5.2.4 on 2025-07-26 00:47

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=255, unique=True)),
                ('first_name', models.CharField(max_length=255)),
                ('last_name', models.CharField(max_length=255)),
                ('is_active', models.BooleanField(default=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('is_superuser', models.BooleanField(default=False)),
                ('is_verified', models.BooleanField(default=False)),
                ('auth_provider', models.CharField(choices=[('email', 'Email'), ('google', 'Google'), ('facebook', 'Facebook')], default='email', max_length=50)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                ('groups', models.ManyToManyField(blank=True, to='auth.group')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'ordering': ['-created_at'],
                'indexes': [models.Index(fields=['email'], name='users_user_email_6f2530_idx')],
            },
        ),
    ]



from setuptools import setup, find_packages
import pathlib

# Read the contents of your README file
BASE_DIR = pathlib.Path(__file__).resolve().parent
README = (BASE_DIR / "README.md").read_text(encoding="utf-8")



setup(
    name="django-auth-boilerplate",
    version="0.1.0",
    author="Emmanuel Ibe",
    author_email="ibehemmanuel32@gmail.com",
    description="A clean Django authentication boilerplate with JWT, rate limiting, and custom exception handling.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/Meekemma/auth-boilerplate.git",
    project_urls={
        "Bug Tracker": "https://github.com/Meekemma/auth-boilerplate.git/issues",
        "Documentation": "https://github.com/Meekemma/auth-boilerplate.git#readme",
    },
    license="MIT",
    packages=find_packages(exclude=["tests", "tests.*", "example_project", "example_project.*"]),
    include_package_data=True,
    install_requires=[
        "Django==5.2.4",
        "djangorestframework==3.16.0",
        "djangorestframework_simplejwt==5.5.1",
        "PyJWT==2.10.1",
        "asgiref==3.9.1",
        "sqlparse==0.5.3",
        "tzdata==2025.2",
    ],
    classifiers=[
        "Framework :: Django",
        "Framework :: Django :: 5.2",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Development Status :: 3 - Alpha",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
    keywords="django authentication jwt rest framework boilerplate",
)



from rest_framework.throttling import ScopedRateThrottle


class LoginThrottle(ScopedRateThrottle):
    scope = 'login'



class RegisterThrottle(ScopedRateThrottle):
    scope = 'register'



class LogoutThrottle(ScopedRateThrottle):
    scope = 'logout'










from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from .models import *
from django.utils.translation import gettext_lazy as _



class UserAdmin(BaseUserAdmin):
    model = User
    ordering = ['-created_at']
    list_display = ['id', 'email', 'first_name', 'last_name','is_active', 'is_staff', 'is_verified', 'is_superuser','auth_provider', 'get_groups_display', 'created_at']
    search_fields = ['id', 'email', 'first_name', 'last_name']
    list_filter = ['is_active', 'is_staff', 'is_superuser']
    
    fieldsets = (
        (None, {'fields': ('email', 'password')}),
        (_('Important dates'), {'fields': ('created_at', 'last_login')}),
        (_('Personal info'), {'fields': ('first_name', 'last_name')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser', 'is_verified', 'groups', 'user_permissions')}),
        (_('Authentication Provider'), {'fields': ('auth_provider',)}),
    )
    
    readonly_fields = ['created_at', 'last_login']

    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'first_name', 'last_name', 'password1', 'password2'),
        }),
    )

    def get_groups_display(self, obj):
        return ", ".join([group.name for group in obj.groups.all()])

    get_groups_display.short_description = 'Groups'






admin.site.register(User, UserAdmin)
